@@ -1,33 +1,44 @@
 use swc_common::{sync::Lrc, SourceMap, DUMMY_SP};
 use swc_ecma_ast::*;
+use swc_ecma_transforms_base::perf::Parallel;
+use swc_ecma_transforms_macros::parallel;
 use swc_ecma_utils::quote_ident;
-use swc_ecma_visit::{noop_fold_type, Fold};
+use swc_ecma_visit::{as_folder, noop_visit_mut_type, Fold, VisitMut};
 
 #[cfg(test)]
 mod tests;
 
 /// `@babel/plugin-transform-react-jsx-source`
-pub fn jsx_src(dev: bool, cm: Lrc<SourceMap>) -> impl Fold {
-    JsxSrc { cm, dev }
+pub fn jsx_src(dev: bool, cm: Lrc<SourceMap>) -> impl Fold + VisitMut {
+    as_folder(JsxSrc { cm, dev })
 }
 
+#[derive(Clone)]
 struct JsxSrc {
     cm: Lrc<SourceMap>,
     dev: bool,
 }
 
-/// TODO: VisitMut
-impl Fold for JsxSrc {
-    noop_fold_type!();
+impl Parallel for JsxSrc {
+    fn create(&self) -> Self {
+        self.clone()
+    }
 
-    fn fold_jsx_opening_element(&mut self, mut e: JSXOpeningElement) -> JSXOpeningElement {
+    fn merge(&mut self, _: Self) {}
+}
+
+#[parallel]
+impl VisitMut for JsxSrc {
+    noop_visit_mut_type!();
+
+    fn visit_mut_jsx_opening_element(&mut self, e: &mut JSXOpeningElement) {
         if !self.dev || e.span == DUMMY_SP {
-            return e;
+            return;
         }
 
         let file_lines = match self.cm.span_to_lines(e.span) {
             Ok(v) => v,
-            _ => return e,
+            _ => return,
         };
 
         e.attrs.push(JSXAttrOrSpread::JSXAttr(JSXAttr {
@@ -61,7 +72,5 @@ impl Fold for JsxSrc {
                 )),
             })),
         }));
-
-        e
     }
 }
